import numpy as np

from ceevee.base import AbstractPredictor


class DummyPredictor(AbstractPredictor):
    """
    The model effectively does nothing
    """

    def __init__(self):
        super().__init__()

    def preprocess(self, x, *kw):
        return x

    def process(self, x, *kw):
        return np.array(x.shape)

    def postprocess(self, x, *kw):
        return x

    def __call__(self, x, *args, **kwargs):
        x = self.preprocess(x)
        x = self.process(x)
        x = self.postprocess(x)
        return {'success': True, 'result': x}



from fire import Fire
from tqdm import tqdm

from ceevee import MODELS
from ceevee.utils import jsonify, read_img


def main(task, *imgs):
    result = []
    model = MODELS[task]()
    for path in tqdm(imgs):
        img = read_img(path)
        pred = model(img)
        pred['path'] = path
        result.append(pred)

    print(jsonify(result))


if __name__ == '__main__':
    Fire(main)



import os
import sys

import falcon
from falcon_multipart.middleware import MultipartMiddleware

from ceevee import MODELS
from ceevee.base import AbstractPredictor
from ceevee.utils import jsonify, read_img


class BaselineServer:
    def __init__(self, model: AbstractPredictor):
        self.model = model

    def _run_model(self, path):
        img = read_img(path)
        result = self.model(img)
        return jsonify(result)

    def _process(self, img, resp):
        fname = img.filename

        with open(fname, 'wb') as out:
            out.write(img.file.read())
        resp.body = self._run_model(fname)
        resp.status = falcon.HTTP_200
        os.remove(fname)

    def on_post(self, req, resp):
        img = req.get_param('image')
        if img is None:
            resp.body = jsonify({'success': False, 'error': 'Image not found'})
            return
        try:
            self._process(img, resp)
        except Exception:
            ex_type, ex_value, ex_traceback = sys.exc_info()
            resp.body = jsonify({'success': False, 'error': f'{ex_type.__name__}: {ex_value}'})


tasks = set(os.environ.get('CEEVEE_TASKS', 'dummy').split(','))
models = {task: MODELS[task]() for task in tasks}
app = falcon.API(middleware=[MultipartMiddleware()])

for task, model in models.items():
    app.add_route(f'/{task}', BaselineServer(model))




import abc

import numpy as np
import torch


class AbstractPredictor:
    @abc.abstractmethod
    def __init__(self):
        pass

    def __call__(self, x, *args, **kwargs) -> dict:
        assert len(x.shape) == 3, "Batch mode is not supported for a while"
        h, w, _ = x.shape
        x = self.preprocess(x)
        res = self.process(x)
        res = self.postprocess(res)
        return res

    @abc.abstractmethod
    def preprocess(self, x, *kw):
        pass

    @abc.abstractmethod
    def process(self, x, *kw):
        pass

    @abc.abstractmethod
    def postprocess(self, x, *kw):
        pass

    @staticmethod
    def _array_to_batch(x):
        x = np.transpose(x, (2, 0, 1))
        x = np.expand_dims(x, 0)
        return torch.from_numpy(x)

    @staticmethod
    def _batch_to_array(x):
        assert x.size()[0] == 1, "Batch size = 1 is only supported for a while"
        x, = x.cpu().numpy()
        x = np.transpose(x, (1, 2, 0))
        return x




from typing import Union

import cv2
import numpy as np
import torch
import ujson as json
import yaml


def load_yaml(x: str):
    with open(x) as fd:
        return yaml.load(fd, yaml.FullLoader)


def ensure_dict(x: Union[str, dict]):
    if isinstance(x, dict):
        return x
    return load_yaml(x)


def to_device(x):
    return x.cuda() if torch.cuda.is_available() else x


def load_jit_model(x):
    return torch.jit.load(x, map_location='cuda:0' if torch.cuda.is_available() else 'cpu')


def read_img(x: str):
    img = cv2.imread(x)
    if img is None:
        raise IOError(f'No such file: `{x}` or file corrupted')

    if len(img.shape) == 3:  # BGR -> RGB
        img = img[:, :, ::-1]
    elif len(img.shape) < 3:  # grayscale -> RGB
        img = np.expand_dims(img, -1)
        img = cv2.cvtColor(img, cv2.COLOR_GRAY2RGB)

    return img


def ensure_serializable(data):
    if isinstance(data, dict):
        return {str(k): ensure_serializable(data[k]) for k in data.keys()}
    elif isinstance(data, list):
        return [ensure_serializable(x) for x in data]
    elif isinstance(data, np.ndarray):
        return data.tolist()
    return data


def jsonify(x):
    return json.dumps(ensure_serializable(x))
